PROJECT_AUTH=\
"""
#Python
from jose import jwt
from pydantic.networks import EmailStr
from passlib.context import CryptContext
from datetime import datetime, timedelta

#FastAPI
from fastapi.security import OAuth2PasswordBearer

#App
from {{project_name}}.settings import TokensConfig

#Database
from database import models
from sqlalchemy.orm import Session


oauth2_scheme = OAuth2PasswordBearer(tokenUrl="/api/v1/login/")
pwd_context = CryptContext(schemes=["bcrypt"], deprecated="auto")

def verify_password(plain_password:str, hashed_password:str):
    return pwd_context.verify(plain_password, hashed_password)


def hash_password(password:str):
    return pwd_context.hash(password)

def create_user_token(data: dict,tokens_config:TokensConfig):
    to_encode = data.copy()
    expire = datetime.utcnow() + timedelta(days=tokens_config.TIME_DELTA)
    scope = "access_token"
    to_encode.update({"exp": expire})
    to_encode.update({"scope": scope})

    encoded_jwt=jwt.encode(to_encode,
                           tokens_config.SECRET_KEY,
                           algorithm=tokens_config.JMV_ALGORITHM)
    return encoded_jwt
"""


import os 
import typer
from pathlib import Path
from kamaqi.utils.files import read_project_file
from kamaqi.utils.files import add_kamaqi_file
from kamaqi.templates.get_templates import get_app_template
from kamaqi.templates.get_templates import get_project_template
from kamaqi.templates.get_templates import get_database_template


app=typer.Typer(help="Create files for your apps")

@app.command(name="apps",
             help="Create files for multiple apps")
def upgrade_apps():

    project_data = read_project_file()
    project_path = Path(project_data["project_path"])
    project_type = project_data["project_type"]
    apps_list=[]
    
    for app, data in project_data["apps"].items():
        if data["status"] == "added":
            apps_list.append(app)

    base_dir_files: Path
    if project_type=="normal":
        base_dir_files=project_path.resolve()
    else:
        base_dir_files=project_path.joinpath("src").resolve()

    for app_name in apps_list:
        base_dir_files.joinpath(app_name).resolve().mkdir()

        for template_name in ["router", "schemas","crud"]:
            template = get_app_template(template_name)
            template_text=template.render(app=app_name[:-1])
            file_path = base_dir_files.joinpath(f"{app_name}/{template_name}.py").resolve()
            file_path.write_text(template_text,encoding="utf-8")
            
    template = get_project_template("main")
    template_text=template.render(**project_data)
    file_path = base_dir_files.joinpath("main.py").resolve()
    file_path.write_text(template_text,encoding="utf-8")

    new_apps = [app[:-1] for app in apps_list]

    template = get_database_template("model")
    models_path = base_dir_files.joinpath("database/models.py").resolve()
    models_text = models_path.read_text()

    for app in new_apps:
        template_text=template.render(app=app)
        models_text+="\n\n"+ template_text
    
    models_path.write_text(models_text,encoding="utf-8")

    for app_name in apps_list:
        project_data["apps"][app_name]["status"]="upgraded"

    project_file_path = project_path.joinpath("kamaqi.json").resolve()

    add_kamaqi_file(project_file_path,project_data)



from typer import Typer
from rich import print
from rich.table import Table
from kamaqi.utils.files import read_project_file

app=Typer(help="Show apps and the project config")

@app.command(name="apps",
             help="Show apps")
def list_apps():

    project_file = read_project_file()
    project_apps = project_file["apps"]
    apps_table = Table("App","Status")

    for app,data in project_apps.items():
        app_status = data["status"].capitalize()
        app_name = app.capitalize()
        if app_status == "Upgraded":
            app_status = f"[bold green]{app_status}[/bold green]"
        if app_status == "Added":
            app_status = f"[bold yellow]{app_status}[/bold yellow]"
            
        apps_table.add_row(app_name,app_status)

    print(apps_table)

@app.command(name="config",
            help="Show kamaqi.json")
def show_config():
    project_file=read_project_file()
    print(project_file)



SCRIPT_PY_MAKO = \
"""
from alembic import op
import sqlalchemy as sa
${imports if imports else ""}

# revision identifiers, used by Alembic.
revision = ${repr(up_revision)}
down_revision = ${repr(down_revision)}
branch_labels = ${repr(branch_labels)}
depends_on = ${repr(depends_on)}


def upgrade() -> None:
    ${upgrades if upgrades else "pass"}


def downgrade() -> None:
    ${downgrades if downgrades else "pass"}
"""


DOCKER_FILE=\
"""
FROM ubuntu

RUN apt update -y
RUN apt install pip -y

WORKDIR /{{project_name}}

COPY ./requirements.txt ./requirements.txt

RUN pip install --no-cache-dir -r requirements.txt

CMD ["uvicorn", "main:app","--reload", "--host", "0.0.0.0", "--port", "80"]

"""



CRUD=\
"""
#Python

#{{app.capitalize()}}
from {{app}}s.schemas import {{app.capitalize()}}Create, {{app.capitalize()}}Update

#Database
from sqlalchemy.orm import Session
from database import models

def insert_{{app}}(db: Session, 
                 {{app}}:{{app.capitalize()}}Create):

    db_{{app}} = models.{{app.capitalize()}}(**{{app}}.dict())
    db.add(db_{{app}})
    db.commit()
    db.refresh(db_{{app}})

    return db_{{app}}

def select_{{app}}_by_id(db: Session, id: int):

    db_{{app}}= db.query(models.{{app.capitalize()}})\\
                .filter(models.{{app.capitalize()}}.id == id)\\
                .first()
                
    return db_{{app}}

def select_all_{{app}}s(db:Session):

    db_{{app}}s= db.query(models.{{app.capitalize()}}).all()
    return db_{{app}}s

def update_{{app}}_in_db(db: Session, id: int, 
                        {{app}}:{{app.capitalize()}}Update):

    db_{{app}} = select_{{app}}_by_id(db, id)

    db_{{app}}.name={{app}}.name
    db.add(db_{{app}})
    db.commit()
    db.refresh(db_{{app}})

    return db_{{app}}

def delete_{{app}}_in_db(db: Session, id: int):
    
    db_user = select_{{app}}_by_id(db, id)

    db.delete(db_user)
    db.commit()
"""


import os 
from typer import prompt

project_types={
    "1": "normal",
    "2": "docker"
    }

def choose_project_type():
    print("Setting your project")
    print("Choose an option")
    print("[1] >> Create a  normal project")
    print("[2] >> Create a project with Docker")
    project_option = prompt("Your option is? ")
    os.system("clear")

    return project_types[project_option]


import os
import typer
from kamaqi.utils.files import read_project_file
from pathlib import Path

app=typer.Typer(help="Update your database")

@app.command(name="tables",
             help="Upgrade your database tables")
def upgrade_tables(message:str=typer.Option(...,"--message","-m")):

    project_data:dict = read_project_file()
    project_name:str = project_data["project_name"]

    if project_data["project_type"] == "normal":
        env_path: Path
        if os.name == "posix":
            env_path = Path("./env/bin/activate").resolve()
            os.system(f". {str(env_path)}  && alembic revision --autogenerate -m'{message}' && alembic upgrade head")
        if os.name == "nt":
            env_path = Path("./env/Scripts/activate").resolve()
            os.system(f"{str(env_path)}  && alembic revision --autogenerate -m'{message}' && alembic upgrade head")


    if project_data["project_type"] == "docker":
        os.system(f"""docker-compose exec {project_name} alembic revision --autogenerate -m"{message}" """)
        os.system(f"docker-compose exec {project_name} alembic upgrade head")


import json
from pathlib import Path

def read_project_file():
    with open("kamaqi.json",mode="r") as f:
        data=json.load(f)
        f.close()
    
    return data

def save_project_file(data:dict):

    with open("kamaqi.json", "w") as outfile:
        json.dump(data, outfile,indent=4)
        outfile.close()

def add_kamaqi_file(path:Path,data:dict):

    with open(path, "w") as outfile:
        json.dump(data, outfile)
        outfile.close()


import os
from rich import print
import typer
from typing import Optional
from pathlib import Path
from kamaqi.utils.files import read_project_file

app = typer.Typer(help="Run your project")

@app.command(name="project",
             help="Run your project")
def run_project(build: Optional[bool] = typer.Option(False, "--build","-b",help="Rebuild the project image")):

    project_file = read_project_file()
    project_name = project_file["project_name"]
    project_type = project_file["project_type"]

    if build and project_type == "normal":
        print("You can't build in normal projects")
        typer.Exit()

    if project_file["project_type"] == "normal":
        env_path: Path
        if os.name == "posix":
            env_path = Path("./env/bin/activate").resolve()
            os.system(f". {str(env_path)} && uvicorn main:app --reload")
        if os.name == "nt":
            env_path = Path("./env/Scripts/activate").resolve()
            os.system(f"{str(env_path)} && uvicorn main:app --reload")
            

    if  project_file["project_type"] == "docker":
        if build:
            os.system(f"sudo docker build -t {project_name.lower()}_image .")
        os.system(f"docker-compose up -d && docker-compose logs -f {project_name}")

    


import os
import secrets
from rich import print
from typer import Typer
from pathlib import Path
from kamaqi.utils.files import add_kamaqi_file
from kamaqi.config.database import choose_database_type
from kamaqi.config.project import choose_project_type
from kamaqi.templates.get_templates import get_project_template
from kamaqi.templates.get_templates import get_database_template
from kamaqi.templates.get_templates import get_migration_template
from kamaqi.templates.get_templates import get_docker_template

app = Typer(help="Start and configure your project")


@app.command(name="project",
             help="Start a project")
def set_project_path(project_name: str):

    print("Setting project path ...")
    if project_name == ".":
        project_path = Path(f"{os.getcwd()}").resolve()
        project_name = Path(project_path).resolve().parts[-1]
    else:
        project_path = Path(f"{os.getcwd()}/{project_name}").resolve()

    project_type = choose_project_type()
    database_type = choose_database_type(project_type)

    secret_key = secrets.token_hex(32)

    project_data = {
        "project_path": str(project_path),
        "project_name": project_name,
        "project_type": project_type,
        "database_type": database_type,
        "secret_key": secret_key,
        "apps": {project_name: {"status": "added"}}
    }

    print("Creating project directories...")
    base_dir_files: Path
    file_path: Path

    if not project_path.exists():
        project_path.mkdir()
    if project_type == 'normal':
        project_path.joinpath(project_name).resolve().mkdir()
        project_path.joinpath("database").resolve().mkdir()
        base_dir_files = project_path
    else:
        project_path.joinpath("db_volume").resolve().mkdir()
        project_path.joinpath("src").resolve().mkdir()
        project_path.joinpath(f"src/{project_name}").resolve().mkdir()
        project_path.joinpath(f"src/database").resolve().mkdir()
        base_dir_files = project_path.joinpath("src").resolve()

    print("Setting environment variables...")
    env_template = get_project_template("env")
    env_text = env_template.render(**project_data)
    file_path = base_dir_files.joinpath(".env").resolve()
    file_path.write_text(env_text, encoding="utf-8")

    print("Creating project files...")
    project_templates = ["auth", "router", "settings", "schemas", "exceptions"]

    for template_name in project_templates:
        template = get_project_template(template_name)
        template_text = template.render(**project_data)
        file_path = base_dir_files.joinpath(f"{project_name}/{template_name}.py").resolve()
        file_path.write_text(template_text, encoding="utf-8")

    print("Setting database ...")
    database_templates = ["database", "models"]
    for template_name in database_templates:
        template = get_database_template(template_name)
        template_text = template.render(**project_data)
        file_path = base_dir_files.joinpath(f"database/{template_name}.py").resolve()
        file_path.write_text(template_text, encoding="utf-8")

    print("Setting database migrations...")
    migrations_templates = ["ini", "env", "script_py_mako"]
    base_dir_files.joinpath("migrations").resolve().mkdir()
    base_dir_files.joinpath("migrations/versions").resolve().mkdir()
    for template_name in migrations_templates:
        template = get_migration_template(template_name)
        template_text = template.render(**project_data)
        if template_name == "ini":
            file_path = base_dir_files.joinpath("alembic.ini").resolve()
        if template_name == "env":
            file_path = base_dir_files.joinpath("migrations/env.py").resolve()
        if template_name == "script_py_mako":
            file_path = base_dir_files.joinpath("migrations/script.py.mako").resolve()
        file_path.write_text(template_text, encoding="utf-8")

    print("Setting python libraries ...")
    template = get_project_template("requirements")
    template_text = template.render(**project_data)
    file_path = project_path.joinpath("requirements.txt").resolve()
    file_path.write_text(template_text, encoding="utf-8")

    template = get_project_template("main")
    template_text = template.render(**project_data)
    file_path = base_dir_files.joinpath("main.py").resolve()
    file_path.write_text(template_text, encoding="utf-8")
    
    project_data["apps"][project_name] = {"status": "upgraded"}
    del project_data["secret_key"]
    project_file_path = project_path.joinpath("kamaqi.json").resolve()
    add_kamaqi_file(project_file_path, project_data)

    os.chdir(project_path)

    print("Setting git config...")

    os.system("git init")
    template = get_project_template("gitignore")
    template_text = template.render(**project_data)
    file_path = project_path.joinpath(".gitignore").resolve()
    file_path.write_text(template_text, encoding="utf-8")
    os.system("git add .gitignore")

    if project_type == "docker":
        print("Creating a docker image...")
        template = get_docker_template("docker_file")
        template_text = template.render(**project_data)
        file_path = project_path.joinpath("Dockerfile").resolve()
        file_path.write_text(template_text, encoding="utf-8")
        template = get_docker_template("docker_compose")
        template_text = template.render(**project_data)
        file_path = project_path.joinpath("docker-compose.yaml").resolve()
        file_path.write_text(template_text, encoding="utf-8")

        try:
            os.system("docker-compose stop")
            os.system("docker container prune --force")
            os.system("docker system prune --force")
            os.system(f"docker image rm {project_name.lower()}_image:latest")
        finally:
            pass

        print("Building  your project image")
        os.system(f"docker build -t {project_name.lower()}_image .")
        os.system(f"docker run {project_name.lower()}_image pip freeze > requirements.txt")
        
    if project_type == "normal":
        print("Creating  a virtual environment ...")

        if os.name == "posix":
            os.system("python3 -m venv env")
            env_path = Path("./env/bin/activate").resolve()
            print("Installing dependencies")
            os.system(f". {str(env_path)} && pip install -r requirements.txt")
            os.system(f". {str(env_path)} && pip freeze > requirements.txt")
            
        if os.name == "nt":
            os.system("py -m venv env")
            env_path = Path("./env/Scripts/activate").resolve()
            print("Installing dependencies")
            os.system(f"{str(env_path)} && pip install -r requirements.txt")
            os.system(f"{str(env_path)} && pip freeze > requirements.txt")

    print(" Yor project was created successfully")



PROJECT_SETTINGS=\
"""
from fastapi_mail import ConnectionConfig
from functools import lru_cache
from pydantic_settings import BaseSettings, SettingsConfigDict
from pydantic import EmailStr

class {{project_name.capitalize()}}Settings(BaseSettings):
    BACKEND_HOST:str
    
    model_config = SettingsConfigDict(env_file='.env', extra='ignore')

@lru_cache()
def get_{{project_name}}_settings():
    return {{project_name.capitalize()}}Settings()


class TokensConfig(BaseSettings):
    SECRET_KEY:str
    JMV_ALGORITHM:str
    TIME_DELTA:int
    
    model_config = SettingsConfigDict(env_file='.env', extra='ignore')

@lru_cache()
def get_tokens_config()->TokensConfig:
    return TokensConfig()

class EmailsSettings(BaseSettings):
    MAIL_USERNAME:str
    MAIL_PASSWORD:str
    MAIL_FROM:EmailStr
    MAIL_FROM_NAME:str
    MAIL_PORT:int
    MAIL_SERVER:str
    MAIL_TLS:bool
    MAIL_SSL:bool
    TEMPLATE_FOLDER:str

    model_config = SettingsConfigDict(env_file='.env', extra='ignore')

@lru_cache()
def get_email_settings():
    email_settings=EmailsSettings()
    return ConnectionConfig(**email_settings.dict())


class DatabaseSettings(BaseSettings):
    DATABASE_USER:str
    DATABASE_PASSWORD:str
    DATABASE_HOST:str
    DATABASE_PORT:str
    DATABASE_NAME:str
    
    model_config = SettingsConfigDict(env_file='.env', extra='ignore')

def get_database_string_connection():
    {% if database_type == "SQLite" %}
    connection_string="sqlite:///./sql_{{project_name}}.db"
    {% else %}
    database_settings=DatabaseSettings()
    {% if database_type == "PostgreSQL" %}
    connection_string="postgresql+psycopg2://user:password@host:port/database_name"
    {% endif %}
    {% if database_type == "MySQL" %}
    connection_string="mysql+pymysql://user:password@host:port/database_name"
    {% endif %}
    connection_string=connection_string\\
                      .replace("user",database_settings.DATABASE_USER)\\
                      .replace("password",database_settings.DATABASE_PASSWORD)\\
                      .replace("host",database_settings.DATABASE_HOST)\\
                      .replace("port",database_settings.DATABASE_PORT)\\
                      .replace("database_name",database_settings.DATABASE_NAME)
    {% endif %}  
    return connection_string
"""


PROJECT_EXCEPTIONS=\
"""
from fastapi import HTTPException,status

different_passwords_exception=HTTPException(
                             status_code=status.HTTP_400_BAD_REQUEST,
                             detail="Different passwords")

credentials_exception = HTTPException(
                        status_code=status.HTTP_401_UNAUTHORIZED,
                        detail="Could not validate credentials incorrect email or password",
                        headers={"WWW-Authenticate": "Bearer"})

access_token_exception=HTTPException(
                       status_code=status.HTTP_401_UNAUTHORIZED,
                       detail="Could not validate credentials,Send access token",
                       headers={"WWW-Authenticate": "Bearer"})

refresh_token_exception=HTTPException(
                        status_code=status.HTTP_401_UNAUTHORIZED,
                        detail="Could not validate credentials,Send refresh token",
                        headers={"WWW-Authenticate": "Bearer"})

email_password_exception=HTTPException(
                         status_code=status.HTTP_401_UNAUTHORIZED,
                         detail="incorrect email or password",
                         headers={"WWW-Authenticate": "Bearer"})


email_not_found_exception=HTTPException(
                          status_code=status.HTTP_401_UNAUTHORIZED,
                          detail="Email not found",
                          headers={"WWW-Authenticate": "Bearer"})


not_registered_user_exception=HTTPException(
                             status_code=status.HTTP_401_UNAUTHORIZED,
                             detail="The user is not registered",
                             headers={"WWW-Authenticate": "Bearer"})

invalid_code_exception=HTTPException(
                       status_code=status.HTTP_401_UNAUTHORIZED,
                       detail="Incorrect code",
                       headers={"WWW-Authenticate": "Bearer"})

inactive_user_exception=HTTPException(
                        status_code=status.HTTP_401_UNAUTHORIZED, 
                        detail="Inactive user")

registered_user_exception=HTTPException(status_code=status.HTTP_400_BAD_REQUEST,
                         detail="The user is already registered")

unauthorized_user_exception=HTTPException(status_code=status.HTTP_401_UNAUTHORIZED,
                           detail="Unauthorized user",
                           headers={"WWW-Authenticate": "Bearer"})
"""


DOCKER_COMPOSE=\
"""
version: "3.8"
services:

  {{project_name}}_db:
    container_name: {{project_name}}_db
    {% if database_type=='PostgreSQL' %}
    image: postgres
    environment:
      POSTGRES_USER: {{project_name}}_user
      POSTGRES_PASSWORD: {{project_name}}_password
      POSTGRES_DB: {{project_name}}_db
    ports:
      - "5432:5432"
    volumes: 
      - ./db_volume:/var/lib/postgresql/data
    {% else %}
    image: mysql
    environment:
      MYSQL_USER: {{project_name}}_user
      MYSQL_PASSWORD: {{project_name}}_password
      MYSQL_ROOT_PASSWORD: {{project_name}}_password
      MYSQL_DATABASE: {{project_name}}_db
    ports:
      - "3306:3306"
    volumes: 
      - ./db_volume:/var/lib/mysql
    {% endif %}

  {{project_name}}:
    container_name: {{project_name}}
    image: {{project_name.lower()}}_image
    volumes:
      - ./src:/{{project_name}}
    ports:
      - "8000:80"
"""



PROJECT_MAIN=\
"""
from fastapi import FastAPI
from fastapi.middleware.cors import CORSMiddleware
# Import routs
{%  for app in apps.keys() %}
from {{app}}.router import {{app}}_routes
{%  endfor %}

app = FastAPI(title="{{project_name.capitalize()}} API",
              description="A powerful API for {{project_name.capitalize()}} project",
              version="1.0.0")

origins = ["*"]

app.add_middleware(
    CORSMiddleware,
    allow_origins=origins,
    allow_credentials=True,
    allow_methods=["*"],
    allow_headers=["*"])

# Include routs
{%  for app in apps.keys() %}
app.include_router({{app}}_routes)
{%  endfor %}
""" 


PROJECT_SCHEMAS=\
"""
#Python
from pydantic import BaseModel, EmailStr
from typing import Optional


class AccessToken(BaseModel):
    access_token: str
    token_type: str


class AccessAndRefreshToken(AccessToken):
    refresh_token: str


class TokenData(BaseModel):
    email: Optional[str] = None


class UpdatePassword(BaseModel):
    new_password: str
    password_confirmation: str


class ForgotPasswordEmail(BaseModel):
    email:EmailStr
"""


import os
from typer import prompt

databases={
    "1": "MySQL",
    "2": "PostgreSQL",
    "3": "SQLite"}

def choose_database_type(project_type):
    print("Setting your project")
    print("Choose an database option")

    possible_dbs=databases.copy()

    if project_type=="docker":
        del possible_dbs["3"]

    for key, vale in possible_dbs.items():
        print(f"[{key}] >> Use {vale}")

    db_option = prompt("Your option is? ")
    os.system("clear")

    return databases[db_option]



SCHEMAS=\
"""
from pydantic import BaseModel, ConfigDict

class {{app.capitalize()}}Create(BaseModel):
    name:str

class {{app.capitalize()}}Read(BaseModel):
    id:int
    name:str
    
    model_config = ConfigDict(from_attributes=True)
    
class {{app.capitalize()}}Update(BaseModel):
    name:str
"""



PROJECT_ENV=\
"""
#app settings
BACKEND_HOST=http://localhost:8000

#auth settings
SECRET_KEY={{secret_key}}
JMV_ALGORITHM=HS256
TIME_DELTA=2

#db config local
{% if database_type!= 'SQLite' %}
DATABASE_USER={{project_name}}_user
DATABASE_PASSWORD={{project_name}}_password
{% if project_type=='normal' %}
DATABASE_HOST=localhost
{% else %}
DATABASE_HOST={{project_name}}_db
{% endif %}
DATABASE_NAME={{project_name}}_db
{% if database_type == 'MySQL' %}
DATABASE_PORT=3306
{% endif %}
{% if database_type == 'PostgreSQL' %}
DATABASE_PORT=5432
{% endif %}
{% endif %}

#db config production
#DATABASE_USER=
#DATABASE_PASSWORD=
#DATABASE_HOST=
#DATABASE_NAME=
#DATABASE_PORT=

#email settings
#MAIL_USERNAME=
#MAIL_PASSWORD=
#MAIL_FROM=
#MAIL_FROM_NAME=
#MAIL_PORT=587
#MAIL_SERVER=smtp.gmail.com
#MAIL_TLS=True
#MAIL_SSL=False
#TEMPLATE_FOLDER=emails/templates 
"""
 


import os 
import typer
from typing import Optional
from kamaqi.utils.files import read_project_file
from kamaqi.start import start
from kamaqi.run import run
from kamaqi.add import add
from kamaqi.show import show
from kamaqi.remove import remove
from kamaqi.upgrade import upgrade
from kamaqi.migrate import migrate
from kamaqi import __version__

app = typer.Typer(help="A command line app for creating Backends with FastAPI")


def version_callback(value: bool):
    if value:
        print(f"Kamaqi version: {__version__}")
        raise typer.Exit()


@app.callback()
def main(version: Optional[bool] = typer.Option(None, "--version", callback=version_callback)):
    if version:
        print(f"Kamaqi version: {__version__}")

@app.command(name="exec",
             help="Execute a command in the project container")
def exec_a_comand(comand: str = typer.Argument("",help="Any Linux comand(Ubuntu).")):

    if comand=="":
        typer.Exit()
    else:
        project_data=read_project_file()
        project_name=project_data["project_name"]
        os.system(f"docker-compose exec {project_name} {comand}")


app.add_typer(start.app, name="start")
app.add_typer(run.app, name="run")
app.add_typer(add.app, name="add")
app.add_typer(show.app, name="show")
app.add_typer(remove.app, name="remove")
app.add_typer(upgrade.app, name="upgrade")
app.add_typer(migrate.app, name="migrate")



PROJECT_REQUIREMENTS=\
"""
python-jose[cryptography]
{% if database_type=='MySQL'%}
pymysql
{% endif %}
{% if database_type=='PostgreSQL'%}
psycopg2-binary
{% endif %}
SQLAlchemy
alembic
passlib[bcrypt]
fastapi[all]
fastapi-mail
"""


PROJECT_ROUTER=\
"""
from fastapi import APIRouter,Depends
from fastapi.responses import RedirectResponse
from {{project_name}}.settings import {{project_name.capitalize()}}Settings,get_{{project_name}}_settings

{{project_name}}_routes= APIRouter()

@{{project_name}}_routes.get(path="/",
                   include_in_schema=False)
def documentation({{project_name}}_settings:{{project_name.capitalize()}}Settings=Depends(get_{{project_name}}_settings)):
    return RedirectResponse(url={{project_name}}_settings.BACKEND_HOST +"/docs/")
"""



import os
from typer import Typer
from typing import List
from kamaqi.utils.files import read_project_file, save_project_file


app=Typer(help="Remove apps and python deps")

@app.command(name="app",
             help="Remove an app")
def remove_app(app_name:str):

    project_data=read_project_file()

    if app_name not in project_data["apps"].keys():
        print(f"App {app_name} not found")
    else:
        del project_data["apps"][app_name]
    save_project_file(project_data)

@app.command(name="apps",
             help="Remove multiple apps")
def remove_apps(apps:List[str]):

    project_data=read_project_file()

    for app_name in apps:
        if app_name not in project_data["apps"].keys():
            print(f"App {app_name} not found")
        else:
            del project_data["apps"][app_name]
    
    save_project_file(project_data)

@app.command(name="dep",
            help="Romeve a python dep")
def remove_dep(dep:str):
    project_data=read_project_file()
    proyect_type = project_data["project_type"]
    project_name = project_data["project_name"]

    if proyect_type == "normal":
        os.system(f". ./env/bin/activate && pip uninstall {dep}")
        os.system(f". ./env/bin/activate && pip freeze > requirements.txt")
    
    if proyect_type == "docker":
        os.system(f"docker-compose exec {project_name} pip3 uninstall {dep}")
        os.system(f"docker-compose exec {project_name} pip3 uninstall > requirements.txt")

@app.command(name="deps",
            help="Remove  multiple python deps")
def add_deps(deps_list:List[str]):
    
    project_data=read_project_file()
    project_type=project_data["project_type"]
    project_name=project_data["project_name"]

    deps_text =" ".join(deps_list)

    if project_type == "normal":
        os.system(f". ./env/bin/activate && pip uninstall {deps_text}")
        os.system(f". ./env/bin/activate && pip freeze > requirements.txt")

    if project_type == "docker":
        os.system(f"docker-compose exec {project_name} pip3 uninstall {deps_text}")
        os.system(f"docker-compose exec {project_name} pip3 freeze > requirements.txt")


ROUTER=\
"""
from typing import List
from fastapi import APIRouter,Depends,status,HTTPException
from fastapi.responses import JSONResponse
from sqlalchemy.orm import Session
from database.database import get_db

from {{app}}s.schemas import {{app.capitalize()}}Create
from {{app}}s.schemas import {{app.capitalize()}}Read
from {{app}}s.schemas import {{app.capitalize()}}Update

from {{app}}s.crud import insert_{{app}},select_{{app}}_by_id
from {{app}}s.crud import select_all_{{app}}s,update_{{app}}_in_db
from {{app}}s.crud import delete_{{app}}_in_db

{{app}}s_routes= APIRouter(prefix="/api/v1/{{app}}s")


@{{app}}s_routes.post(path="/create/",
                     tags=["{{app.capitalize()}}s"],
                     response_model={{app.capitalize()}}Read,
                     status_code=status.HTTP_201_CREATED)
async def create_{{app}}({{app}}_data:{{app.capitalize()}}Create,
                        db:Session=Depends(get_db)):

    return insert_{{app}}(db,{{app}}_data)

@{{app}}s_routes.get(path="/get/all/",
                    tags=["{{app.capitalize()}}s"],
                    response_model=List[{{app.capitalize()}}Read],
                    status_code=status.HTTP_200_OK)
async def get_all_{{app}}s(db:Session=Depends(get_db)):

    db_{{app}}s=select_all_{{app}}s(db)
    if not db_{{app}}s:
        raise HTTPException(status_code=status.HTTP_404_NOT_FOUND, 
                            detail="{{app}}s not found")
    
    return db_{{app}}s

@{{app}}s_routes.get(path="/get/{id}/",
                    tags=["{{app.capitalize()}}s"],
                    response_model={{app.capitalize()}}Read,
                    status_code=status.HTTP_200_OK)
async def get_{{app}}(id:int,db:Session=Depends(get_db)):

    db_{{app}}=select_{{app}}_by_id(db,id)
    if not db_{{app}}:
        raise HTTPException(status_code=status.HTTP_404_NOT_FOUND, 
                            detail="{{app}} not found")
    return db_{{app}}

@{{app}}s_routes.put(path="/update/{id}/",
                tags=["{{app.capitalize()}}s"],
                response_model={{app.capitalize()}}Read,
                status_code=status.HTTP_200_OK)
async def update_{{app}}(id:int,
                        {{app}}_data:{{app.capitalize()}}Update,
                        db:Session=Depends(get_db)):

    return update_{{app}}_in_db(db,id,{{app}}_data)

@{{app}}s_routes.delete(path="/delete/{id}/",
                    tags=["{{app.capitalize()}}s"],
                    status_code=status.HTTP_200_OK)
async def delete_{{app}}(id:int,db:Session=Depends(get_db)):

    delete_{{app}}_in_db(db,id)

    return JSONResponse(content={"detail":"{{app}} deleted"},
                        status_code=status.HTTP_200_OK)

"""



import os
from typer import Typer
from rich import print
from typing import List

from kamaqi.utils.files import read_project_file,save_project_file

app=Typer(help="Add apps and python deps")

@app.command(name="app",
             help="Add an app")
def create_app(app_name:str):

    project_data=read_project_file()
    project_apps_names=project_data["apps"].keys()

    if app_name in project_apps_names:
        print(f"App {app_name} already exists")
    else:
        project_data["apps"][app_name] = {"status":"added"}

    save_project_file(project_data)

@app.command(name="apps",
            help="Add multiple apps")
def create_apps(apps:List[str]):

    project_data=read_project_file()
    project_apps_names=project_data["apps"].keys()

    for app_name in apps:
        if app_name in project_apps_names:
            print(f"App {app_name} already exists")
        else:
             project_data["apps"][app_name] = {"status":"added"}
    
    save_project_file(project_data)

@app.command(name="dep",
            help="Add a python dep")
def add_dep(dep:str):
    
    project_data=read_project_file()
    project_type=project_data["project_type"]
    project_name=project_data["project_name"]

    if project_type == "normal":
        os.system(f". ./env/bin/activate && pip install {dep}")
        os.system(f". ./env/bin/activate && pip freeze > requirements.txt")

    if project_type == "docker":
        os.system(f"docker-compose exec {project_name} pip3 install {dep}")
        os.system(f"docker-compose exec {project_name} pip3 freeze > requirements.txt")

@app.command(name="deps",
            help="Add a multiple python deps")
def add_deps(deps_list:List[str]):
    
    project_data=read_project_file()
    project_type=project_data["project_type"]
    project_name=project_data["project_name"]

    deps_text =" ".join(deps_list)

    if project_type == "normal":
        os.system(f". ./env/bin/activate && pip install {deps_text}")
        os.system(f". ./env/bin/activate && pip freeze > requirements.txt")

    if project_type == "docker":
        os.system(f"docker-compose exec {project_name} pip3 install {deps_text}")
        os.system(f"docker-compose exec {project_name} pip3 freeze > requirements.txt")
  







APP_DATABASE=\
"""
from sqlalchemy import create_engine
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import sessionmaker
from {{project_name}}.settings import get_database_string_connection


DATABASE_URL=get_database_string_connection()
{% if database_type=='SQLite' %}
engine = create_engine(DATABASE_URL,connect_args={"check_same_thread": False})
{% else %}
engine = create_engine(DATABASE_URL)
{% endif %}

SessionLocal = sessionmaker(autocommit=False, autoflush=False, bind=engine)

Base = declarative_base()


def get_db():
    db = SessionLocal()
    try:
        yield db
    finally:
        db.close()

"""


from jinja2 import Environment, BaseLoader

from kamaqi.templates.project import project_requirements,project_env
from kamaqi.templates.project import project_auth, project_exceptions
from kamaqi.templates.project import project_schemas, project_settings
from kamaqi.templates.project import project_router,project_main
from kamaqi.templates.project import project_gitignore

from kamaqi.templates.database import database, model,models

from kamaqi.templates.migrations import migrations_env, migrations_ini
from kamaqi.templates.migrations import migrations_script_py_mako

from kamaqi.templates.docker import docker_compose,docker_file

from kamaqi.templates.app import app_crud, app_router, app_schemas

environment = Environment(loader=BaseLoader)

def get_project_template(template_name:str):
    
    templates_dict={
        "requirements":project_requirements.PROJECT_REQUIREMENTS,
        "router":project_router.PROJECT_ROUTER,
        "settings":project_settings.PROJECT_SETTINGS,
        "exceptions":project_exceptions.PROJECT_EXCEPTIONS,
        "schemas":project_schemas.PROJECT_SCHEMAS,
        "auth":project_auth.PROJECT_AUTH,
        "env":project_env.PROJECT_ENV,
        "main":project_main.PROJECT_MAIN,
        "gitignore":project_gitignore.PROJECT_GITIGNORE,
    }

    template=environment.from_string(templates_dict[template_name])
    return template

def get_database_template(template_name:str):
    
    templates_dict={
        "database":database.APP_DATABASE,
        "models":models.MODELS,
        "model":model.MODEL,

    }

    template=environment.from_string(templates_dict[template_name])
    return template

def get_migration_template(template_name:str):
    
    templates_dict={
        "ini":migrations_ini.ALEMBIC_INI,
        "env":migrations_env.MIGRATIONS_ENV,
        "script_py_mako":migrations_script_py_mako.SCRIPT_PY_MAKO,
    }

    template=environment.from_string(templates_dict[template_name])
    return template

def get_docker_template(template_name:str):

    templates_dict={
        "docker_file":docker_file.DOCKER_FILE,
        "docker_compose":docker_compose.DOCKER_COMPOSE,

    }

    template=environment.from_string(templates_dict[template_name])
    return template

def get_app_template(template_name:str):
    
    templates_dict={
        "router":app_router.ROUTER,
        "schemas":app_schemas.SCHEMAS,
        "crud":app_crud.CRUD,
    }

    template=environment.from_string(templates_dict[template_name])
    return template